----------------------------------------------------------------------------------
-- Company: UNIVERSITY OF CONNECTICUT
-- Engineer: John A. Chandy
-- 
-- Create Date:    14:06:00 09/29/2006 
-- Design Name: 
-- Module Name:    oneshot - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: This module takes an incoming signal and pulses it
--					 for just one clock cycle.  Useful when taking
--					 external signals such as button presses and making
--					 sure the button press registers only for one clock
--					 cycle.  The signal must be debounced before giving
--					 to this oneshot module.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity oneshot is
    Port ( clk : in  STD_LOGIC;
			  reset : in STD_LOGIC;
           input : in  STD_LOGIC;
           output : out  STD_LOGIC);
end oneshot;

architecture Behavioral of oneshot is

	type state_type is (START, ONESHOT, WAIT4ZERO);
	signal state : state_type;

begin

	-- this state machine basically has three states
	-- START - wait for the input to go high
	-- ONESHOT - set the output to high and go to WAIT4ZERO 
	--           if input is still high
	-- WAIT4ZERO - wait for the input to go back low and
	--             then go back to START
	-- the output is set only for one clock cycle while
	-- the state machine is in the ONESHOT state

	process( clk, reset )
	begin
		if ( reset = '1' ) then
			state <= START;

		elsif ( clk='1' and clk'event) then

			case state is
			when START =>
				if ( input = '1' ) then
					state <= ONESHOT;
				end if;

			when ONESHOT =>
				if ( input = '0' ) then
					state <= START;
				else
					state <= WAIT4ZERO;
				end if;

			when WAIT4ZERO =>
				if ( input = '0' ) then
					state <= START;
				end if;

			when others =>
				state <= START;

			end case;
		end if;
			
	end process;

	output <= '1' when state = ONESHOT else '0';

end Behavioral;

